==> extruct-0.6.0.tar.gz: extruct-0.6.0/extruct/w3cmicrodata.py <==
"""
HTML Microdata parser

Piece of code extracted form:
* http://blog.scrapinghub.com/2014/06/18/extracting-schema-org-microdata-using-scrapy-selectors-and-xpath/

Ported to lxml
follows http://www.w3.org/TR/microdata/#json

"""

import collections
try:
    from urlparse import urljoin
except ImportError:
    from urllib.parse import urljoin

import lxml.etree
import lxml.html
from w3lib.html import strip_html5_whitespace


class LxmlMicrodataExtractor(object):
    _xp_item = lxml.etree.XPath('descendant-or-self::*[@itemscope]')
    _xp_prop = lxml.etree.XPath("""set:difference(.//*[@itemprop],
                                                  .//*[@itemscope]//*[@itemprop])""",
                                namespaces = {"set": "http://exslt.org/sets"})
    _xp_clean_text = lxml.etree.XPath('descendant-or-self::*[not(self::script or self::style)]/text()')
    # ancestor and preceding axes contain all elements before the context node
    # so counting them gives the "document order" of the context node
    _xp_item_docid = lxml.etree.XPath("""count(preceding::*[@itemscope])
                                       + count(ancestor::*[@itemscope])
                                       + 1""")

    def __init__(self, nested=True, strict=False, add_text_content=False, add_html_node=False):
        self.nested = nested
        self.strict = strict
        self.add_text_content = add_text_content
        self.add_html_node = add_html_node

    def get_docid(self, node):
        return int(self._xp_item_docid(node))

    def extract(self, htmlstring, base_url=None, encoding="UTF-8"):
        parser = lxml.html.HTMLParser(encoding=encoding)
        lxmldoc = lxml.html.fromstring(htmlstring, parser=parser)
        return self.extract_items(lxmldoc, base_url)

    def extract_items(self, document, base_url):
        items_seen = set()
        return [
            item for item in (
                self._extract_item(it, items_seen=items_seen, base_url=base_url)
                for it in self._xp_item(document))
            if item]

    def _extract_item(self, node, items_seen, base_url):
        itemid = self.get_docid(node)

        if self.nested:
            if itemid in items_seen:
                return
            items_seen.add(itemid)

        item = {}
        if not self.nested:
            item["iid"] = itemid
        types = node.get('itemtype', '').split()
        if types:
            if not self.strict and len(types) == 1:
                item["type"] = types[0]
            else:
                item["type"] = types

            itemid = node.get('itemid')
            if itemid:
                item["id"] = itemid.strip()

        properties = collections.defaultdict(list)
        # start with item references
        refs = node.get('itemref', '').split()
        if refs:
            for refid in refs:
                for name, value in self._extract_property_refs(
                        node, refid, items_seen=items_seen, base_url=base_url):
                    properties[name].append(value)

        for name, value in self._extract_properties(
                node, items_seen=items_seen, base_url=base_url):
            properties[name].append(value)

        props = []
        for (name, values) in properties.items():
            if not self.strict and len(values) == 1:
                props.append((name, values[0]))
            else:
                props.append((name, values))
        if props:
            item["properties"] = dict(props)
        else:
            # item without properties; let's use the node itself
            item["value"] = self._extract_property_value(
                node, force=True, items_seen=items_seen, base_url=base_url)

        # below are not in the specs, but can be handy
        if self.add_text_content:
            textContent = self._extract_textContent(node)
            if textContent:
                item["textContent"] = textContent
        if self.add_html_node:
            item["htmlNode"] = node

        return item

    def _extract_properties(self, node, items_seen, base_url):
        for prop in self._xp_prop(node):
            for p, v in self._extract_property(
                    prop, items_seen=items_seen, base_url=base_url):
                yield p, v

    def _extract_property_refs(self, node, refid, items_seen, base_url):
        for prop in node.xpath("id($refid)/descendant-or-self::*[@itemprop]", refid=refid):
            for p, v in self._extract_property(
                    prop, items_seen=items_seen, base_url=base_url):
                yield p, v

    def _extract_property(self, node, items_seen, base_url):
        props = node.get("itemprop").split()
        value = self._extract_property_value(
            node, items_seen=items_seen, base_url=base_url)
        return [(p, value) for p in props]

    def _extract_property_value(self, node, items_seen, base_url, force=False):
        #http://www.w3.org/TR/microdata/#values
        if not force and node.get("itemscope") is not None:
            if self.nested:
                return self._extract_item(
                    node, items_seen=items_seen, base_url=base_url)
            else:
                return {"iid_ref": self.get_docid(node)}

        elif node.tag == "meta":
            return node.get("content", "")

        elif node.tag in ("audio", "embed", "iframe", "img", "source", "track", "video"):
            return urljoin(base_url, strip_html5_whitespace(node.get("src", "")))

        elif node.tag in ("a", "area", "link"):
            return urljoin(base_url, strip_html5_whitespace(node.get("href", "")))

        elif node.tag in ("object",):
            return urljoin(base_url, strip_html5_whitespace(node.get("data", "")))

        elif node.tag in ("data", "meter"):
            return node.get("value", "")

        elif node.tag in ("time",):
            return node.get("datetime", "")

        # not in W3C specs but used in schema.org examples
        elif node.get("content"):
            return node.get("content")

        else:
            return self._extract_textContent(node)

    def _extract_textContent(self, node):
        return u"".join(self._xp_clean_text(node)).strip()


MicrodataExtractor = LxmlMicrodataExtractor

==> extruct-0.7.1.tar.gz: extruct-0.7.1/extruct/w3cmicrodata.py <==
"""
HTML Microdata parser

Piece of code extracted form:
* http://blog.scrapinghub.com/2014/06/18/extracting-schema-org-microdata-using-scrapy-selectors-and-xpath/

Ported to lxml
follows http://www.w3.org/TR/microdata/#json

"""

import collections
try:
    from urlparse import urljoin
except ImportError:
    from urllib.parse import urljoin

import lxml.etree
from w3lib.html import strip_html5_whitespace

from extruct.utils import parse_html


class LxmlMicrodataExtractor(object):
    _xp_item = lxml.etree.XPath('descendant-or-self::*[@itemscope]')
    _xp_prop = lxml.etree.XPath("""set:difference(.//*[@itemprop],
                                                  .//*[@itemscope]//*[@itemprop])""",
                                namespaces = {"set": "http://exslt.org/sets"})
    _xp_clean_text = lxml.etree.XPath('descendant-or-self::*[not(self::script or self::style)]/text()')
    # ancestor and preceding axes contain all elements before the context node
    # so counting them gives the "document order" of the context node
    _xp_item_docid = lxml.etree.XPath("""count(preceding::*[@itemscope])
                                       + count(ancestor::*[@itemscope])
                                       + 1""")

    def __init__(self, nested=True, strict=False, add_text_content=False, add_html_node=False):
        self.nested = nested
        self.strict = strict
        self.add_text_content = add_text_content
        self.add_html_node = add_html_node

    def get_docid(self, node):
        return int(self._xp_item_docid(node))

    def extract(self, htmlstring, base_url=None, encoding="UTF-8"):
        tree = parse_html(htmlstring, encoding=encoding)
        return self.extract_items(tree, base_url)

    def extract_items(self, document, base_url):
        items_seen = set()
        return [
            item for item in (
                self._extract_item(it, items_seen=items_seen, base_url=base_url)
                for it in self._xp_item(document))
            if item]

    def _extract_item(self, node, items_seen, base_url):
        itemid = self.get_docid(node)

        if self.nested:
            if itemid in items_seen:
                return
            items_seen.add(itemid)

        item = {}
        if not self.nested:
            item["iid"] = itemid
        types = node.get('itemtype', '').split()
        if types:
            if not self.strict and len(types) == 1:
                item["type"] = types[0]
            else:
                item["type"] = types

            itemid = node.get('itemid')
            if itemid:
                item["id"] = itemid.strip()

        properties = collections.defaultdict(list)
        # start with item references
        refs = node.get('itemref', '').split()
        if refs:
            for refid in refs:
                for name, value in self._extract_property_refs(
                        node, refid, items_seen=items_seen, base_url=base_url):
                    properties[name].append(value)

        for name, value in self._extract_properties(
                node, items_seen=items_seen, base_url=base_url):
            properties[name].append(value)

        props = []
        for (name, values) in properties.items():
            if not self.strict and len(values) == 1:
                props.append((name, values[0]))
            else:
                props.append((name, values))
        if props:
            item["properties"] = dict(props)
        else:
            # item without properties; let's use the node itself
            item["value"] = self._extract_property_value(
                node, force=True, items_seen=items_seen, base_url=base_url)

        # below are not in the specs, but can be handy
        if self.add_text_content:
            textContent = self._extract_textContent(node)
            if textContent:
                item["textContent"] = textContent
        if self.add_html_node:
            item["htmlNode"] = node

        return item

    def _extract_properties(self, node, items_seen, base_url):
        for prop in self._xp_prop(node):
            for p, v in self._extract_property(
                    prop, items_seen=items_seen, base_url=base_url):
                yield p, v

    def _extract_property_refs(self, node, refid, items_seen, base_url):
        for prop in node.xpath("id($refid)/descendant-or-self::*[@itemprop]", refid=refid):
            for p, v in self._extract_property(
                    prop, items_seen=items_seen, base_url=base_url):
                yield p, v

    def _extract_property(self, node, items_seen, base_url):
        props = node.get("itemprop").split()
        value = self._extract_property_value(
            node, items_seen=items_seen, base_url=base_url)
        return [(p, value) for p in props]

    def _extract_property_value(self, node, items_seen, base_url, force=False):
        #http://www.w3.org/TR/microdata/#values
        if not force and node.get("itemscope") is not None:
            if self.nested:
                return self._extract_item(
                    node, items_seen=items_seen, base_url=base_url)
            else:
                return {"iid_ref": self.get_docid(node)}

        elif node.tag == "meta":
            return node.get("content", "")

        elif node.tag in ("audio", "embed", "iframe", "img", "source", "track", "video"):
            return urljoin(base_url, strip_html5_whitespace(node.get("src", "")))

        elif node.tag in ("a", "area", "link"):
            return urljoin(base_url, strip_html5_whitespace(node.get("href", "")))

        elif node.tag in ("object",):
            return urljoin(base_url, strip_html5_whitespace(node.get("data", "")))

        elif node.tag in ("data", "meter"):
            return node.get("value", "")

        elif node.tag in ("time",):
            return node.get("datetime", "")

        # not in W3C specs but used in schema.org examples
        elif node.get("content"):
            return node.get("content")

        else:
            return self._extract_textContent(node)

    def _extract_textContent(self, node):
        return u"".join(self._xp_clean_text(node)).strip()


MicrodataExtractor = LxmlMicrodataExtractor
